import { history, Access, useAccess } from 'umi';
import type { PropsWithChildren } from 'react';
import React, { useEffect, useRef, useState } from 'react';
import type { ActionType, ProColumns } from '@ant-design/pro-table';
import moment from 'moment/moment';
import { PageContainer, ProTable } from '@ant-design/pro-components';
import type { FormInstance } from 'antd';
import { Badge, Dropdown, Menu, message, Switch } from 'antd';
import { Button } from 'antd';
import { DownOutlined, PlusOutlined } from '@ant-design/icons';
import { FormattedMessage } from '@@/plugin-locale/localeExports';
import AddUser from './components/add';
import { Link } from 'umi';
import { disableSwitchUser, identityUser, userList } from '../service';
import type { UserResult } from '../data';

const handleSuperSwitch = async (identity: number, fields: UserResult) => {
  const hide = message.loading('正在修改');
  try {
    const { code, msg } = await identityUser(fields.user_id, {
      identity,
    });
    if (code === 1) {
      hide();
      message.success(msg);
      return true;
    }
    message.error(msg);
    return false;
  } catch (error) {
    hide();
    message.error('Adding failed, please try again!');
    return false;
  }
};

const handleDisableSwitch = async (is: boolean, fields: UserResult) => {
  const hide = message.loading('正在修改');
  try {
    const { code, msg } = await disableSwitchUser(fields.user_id, {
      disable_switch: is ? 1 : 0,
    });
    if (code === 1) {
      hide();
      message.success(msg);
      return true;
    }
    message.error(msg);
    return false;
  } catch (error) {
    hide();
    message.error('Adding failed, please try again!');
    return false;
  }
};

const UserList: React.FC<PropsWithChildren<any>> = (props) => {
  const routeUrl = props.match.url;
  const [searchParams, setSearchParams] = useState(props.location.state?.searchParams || {});

  const actionRef = useRef<ActionType>();
  const searchRef = useRef<FormInstance>();
  useEffect(() => {
    history.replace(routeUrl, { searchParams });
    searchRef.current?.setFieldsValue({
      ...searchParams,
    });
  }, [routeUrl, searchParams]);

  const access = useAccess();
  const [createModalVisible, handleModalVisible] = useState<boolean>(false);

  const superSwitchMenu = (fields: UserResult) => {
    return (
      <Menu
        items={[
          {
            label: <Badge status="default" text="普通用户" />,
            key: 0,
            onClick: async () => {
              const success = await handleSuperSwitch(1, fields);
              if (success) {
                actionRef.current?.reload();
              }
            },
          },
          {
            label: <Badge status="success" text="超级用户" />,
            key: 1,
            onClick: async () => {
              const success = await handleSuperSwitch(2, fields);
              if (success) {
                actionRef.current?.reload();
              }
            },
          },
        ]}
      />
    );
  };
  const columns: ProColumns<UserResult>[] = [
    {
      title: '模糊查询账号',
      hideInTable: true,
      dataIndex: 'keyWords',
    },
    { title: 'ID', dataIndex: 'user_id', search: false },
    { title: '账号', dataIndex: 'account' },
    { title: '姓名', dataIndex: 'name' },
    {
      title: '身份',
      dataIndex: 'identity',
      key: 'identity',
      align: 'center',
      width: 120,
      valueEnum: {
        1: { text: '普通用户', status: 'default' },
        2: { text: '超级用户', status: 'success' },
      },
      render: (text, entity) => (
        <Dropdown overlay={superSwitchMenu(entity)}>
          <span>
            {entity.identity === 1 && <Badge status="default" text="普通用户" />}
            {entity.identity === 2 && <Badge status="success" text="超级用户" />}
            <DownOutlined />
          </span>
        </Dropdown>
      ),
      search: false,
    },
    {
      title: '禁用',
      dataIndex: 'disable_switch',
      key: 'disable_switch',
      align: 'center',
      width: 80,
      render: (text, entity) => (
        <Switch
          checked={entity.disable_switch === 1}
          onChange={async (checked) => {
            const success = await handleDisableSwitch(checked, entity);
            if (success) {
              actionRef.current?.reload();
            }
          }}
        />
      ),
      search: false,
    },
    {
      title: '创建时间',
      dataIndex: 'create_time',
      search: false,
      render: (val) =>
        typeof val === 'number' && val > 0 ? moment(val * 1000).format('YYYY-MM-DD HH:mm:ss') : '-',
    },
    {
      title: '操作',
      dataIndex: 'option',
      valueType: 'option',
      align: 'center',
      fixed: window.innerWidth < 596 ? false : 'right',
      render: (_, record) => [
        <Link
          key="edit"
          to={{ pathname: `/user/settings/${record.user_id}`, state: { searchParams } }}
        >
          编辑
        </Link>,
      ],
    },
  ];
  return (
    <PageContainer>
      <ProTable<UserResult, API.PageParams>
        headerTitle="运营账户"
        scroll={{ x: 1300 }}
        actionRef={actionRef}
        formRef={searchRef}
        rowKey="user_id"
        search={{
          labelWidth: 'auto',
        }}
        pagination={{
          defaultCurrent: searchParams.current || 1,
          defaultPageSize: searchParams.pageSize || 10,
          showSizeChanger: true,
          pageSizeOptions: ['10', '20', '50', '100'],
        }}
        toolBarRender={() => [
          <Access key="new" accessible={access.buttonFilter('/user/list::new')}>
            <Button
              type="primary"
              onClick={() => {
                handleModalVisible(true);
              }}
            >
              <PlusOutlined /> <FormattedMessage id="pages.searchTable.new" defaultMessage="New" />
            </Button>
          </Access>,
        ]}
        request={async (params, sort) => {
          const d = {
            ...searchParams,
            current: params.current,
            pageSize: params.pageSize,
          };
          setSearchParams({ ...d });
          return userList(d, sort).then(({ data }) => {
            return {
              data: data?.data || [],
              success: true,
              total: data?.total || 0,
            };
          });
        }}
        columns={columns}
        onSubmit={(params) => {
          setSearchParams({ ...searchParams, ...params, current: 1 });
        }}
        onReset={() => {
          setSearchParams({});
        }}
      />
      <AddUser
        state={createModalVisible}
        onChange={(visible: boolean) => handleModalVisible(visible)}
        onSubmit={() => {
          actionRef.current?.reload();
        }}
      />
    </PageContainer>
  );
};
export default UserList;
