import { history } from 'umi';
import React, { useEffect, useRef, useState } from 'react';
import {
  DeleteOutlined,
  InboxOutlined,
  LoadingOutlined,
  PaperClipOutlined,
  WarningOutlined,
} from '@ant-design/icons';
import type { UploadProps } from 'antd';
import { message, Upload, Progress, Button, Tooltip } from 'antd';
import { PageContainer, ProCard } from '@ant-design/pro-components';
import type { RcFile } from 'antd/lib/upload';
import type { UploadFileStatus } from 'antd/lib/upload/interface';
import SparkMD5 from 'spark-md5';
import superagent from 'superagent';
import renderSize from '@/global';
import { addUploadTask } from '../service';
import type { UploadTaskAddParams } from '../data';
import CustomizeConfig from '@/services/customizeConfig';

const { Dragger } = Upload;
const { APIDomain } = CustomizeConfig;

type WorkStatus = 'wait' | 'march' | 'uploading' | 'finish';
type WorkData = {
  uid: string;
  name: string;
  status: UploadFileStatus;
  md5?: string;
  size?: number;
};
type UploadParams = { uid: string; file: Record<string, any>; chunks: Record<string, any>[] };

const handleAddUploadTask = async (fields: UploadTaskAddParams) => {
  const hide = message.loading('正在添加任务');
  try {
    const { data, code, msg } = await addUploadTask({ ...fields });
    if (code === 1) {
      hide();
      return data?.ut_id || null;
    }
    message.error(msg);
    return null;
  } catch (error) {
    hide();
    message.error('Adding failed, please try again!');
    return null;
  }
};

const FragmentUpload: React.FC = () => {
  // 可以用来终止计算文件 MD5
  const preStatusRef = useRef(false);
  // 上传任务状态
  const workStatusList = useRef<
    {
      uid: string;
      status: WorkStatus;
      preProgress: string;
      uploadProgress: string;
      uploadStep: number;
      data: WorkData;
      file: RcFile;
      ut_id?: number;
      params?: UploadParams;
    }[]
  >([]);
  // 上传任务列表
  const [workList, setWorkList] = useState<
    {
      uid: string;
      data: WorkData;
    }[]
  >([]);
  useEffect(() => {
    if (workStatusList.current.length == 0) return;
    for (let i = 0; i < workStatusList.current.length; i++) {
      // 有正在计算的，就跳过新任务
      if (['march', 'uploading'].indexOf(workStatusList.current[i].status) != -1) return;
    }
    for (let i = 0; i < workStatusList.current.length; i++) {
      // 有正在计算的，就跳过新任务
      if (workStatusList.current[i].status == 'wait') {
        workStatusList.current[i].status = 'march';
        work(workStatusList.current[i].file);
        return;
      }
    }
  }, [workList]);

  const [uploadPace, setUploadPace] = useState(0);
  useEffect(() => {
    if (workStatusList.current.length == 0) return;
    const list = workStatusList.current.filter((it) => it.status == 'uploading');
    if (list.length == 0) return;
    const task = list[0];
    const paramObject = task.params!!;
    const chunkInfo = paramObject.chunks[task.uploadStep];
    if (!chunkInfo) {
      workStatusList.current = workStatusList.current.map((it) => {
        if (it.uid == task.uid) {
          it.status = 'finish';
          it.uploadProgress = '100';
          it.data.status = 'done';
        }
        return it;
      });
      setWorkList([
        ...workStatusList.current.map((it) => {
          return { uid: it.uid, data: it.data };
        }),
      ]);
      return;
    }
    const token = localStorage.getItem('resource_token');
    const params =
      `token=${token}&fileMd5=${paramObject.file.fileMd5}&chunks=${paramObject.chunks.length}` +
      `&ut_id=${task.ut_id}&chunk=${chunkInfo.chunk}&start=${chunkInfo.start}&end=${chunkInfo.end}&md5=${chunkInfo.chunkMd5}`;
    const blob = new Blob([chunkInfo.currentBuffer], { type: 'application/octet-stream' });
    const formData = new FormData();
    formData.append('chunk', blob, chunkInfo.chunkMd5);
    superagent
      .post(`${APIDomain}/api/upload/addChunk?${params}`)
      .send(formData)
      .end((err: Record<string, any>, res: Record<string, any>) => {
        if (res.body.code != 1) console.log('res.body', res.body);
        workStatusList.current = workStatusList.current.map((it) => {
          if (it.uid == task.uid) {
            it.uploadProgress = ((it.uploadStep / paramObject.chunks.length) * 100).toFixed(2);
            it.uploadStep = it.uploadStep + 1;
          }
          return it;
        });
        setWorkList([
          ...workStatusList.current.map((it) => {
            return { uid: it.uid, data: it.data };
          }),
        ]);
        setUploadPace(1 - uploadPace);
      });
  }, [uploadPace]);

  // 重置
  const reset = () => {
    preStatusRef.current = false;
    workStatusList.current = [];
    setWorkList([]);
  };

  const props: UploadProps = {
    accept: '.mp4,video/mp4,.mov,video/mov',
    multiple: true,
    fileList: workList.map((it) => it.data),
    itemRender: (originNode, file, fileList, actions) => {
      let fileNode;
      if (file.status == 'uploading')
        fileNode = (
          <>
            <LoadingOutlined /> {file.name}
          </>
        );
      else if (file.status == 'done')
        fileNode = (
          <>
            <PaperClipOutlined /> {file.name}
          </>
        );
      else
        fileNode = (
          <>
            <WarningOutlined /> {file.name}
          </>
        );

      const tmpWork = workStatusList.current.filter((it) => {
        return it.uid == file.uid;
      })[0];
      if (!tmpWork) return null;

      return (
        <ProCard
          title={
            <>
              {fileNode} {renderSize(file.size || '')}
            </>
          }
          extra={
            <Tooltip title="search">
              <Button
                type="dashed"
                shape="circle"
                icon={<DeleteOutlined />}
                onClick={actions.remove}
              />
            </Tooltip>
          }
        >
          <div>MD5: {tmpWork.data.md5 && <>{tmpWork.data.md5}</>}</div>
          <div style={{ width: '80%' }}>
            <Progress
              percent={Number(tmpWork.preProgress)}
              success={{ percent: Number(tmpWork.uploadProgress) }}
            />
          </div>
        </ProCard>
      );
    },
    onRemove: (file) => {
      workStatusList.current = workStatusList.current.filter((it) => {
        return it.uid != file.uid;
      });
      setWorkList(
        workStatusList.current.map((it) => {
          return { uid: it.uid, data: it.data };
        }),
      );
    },
    beforeUpload: (file, FileList) => {
      const list = FileList.filter((it) => {
        return !!it.uid;
      });
      if (list.length != FileList.length) return;
      workStatusList.current = [
        ...workStatusList.current,
        ...FileList.map((it) => {
          return {
            uid: it.uid,
            status: 'wait' as WorkStatus,
            preProgress: '0',
            uploadProgress: '0',
            uploadStep: 0,
            data: {
              uid: it.uid,
              name: it.name,
              size: it.size,
              status: 'uploading' as UploadFileStatus,
            },
            file: it,
          };
        }),
      ];
      setWorkList(
        workStatusList.current.map((it) => {
          return { uid: it.uid, data: it.data };
        }),
      );
      return false;
    },
  };

  function work(file: RcFile) {
    preStatusRef.current = true;
    // 上传文件所有分片的 md5
    const params: UploadParams = {
      uid: file.uid,
      file: {},
      chunks: [],
    };
    params.file.fileName = file.name;
    params.file.fileSize = file.size;

    // 分片每片大小 5M
    const chunkSize = 1024 * 1024 * 5;
    // 分片数量
    const chunks = Math.ceil(file.size / chunkSize);
    // 当前分片序号
    let currentChunk = 0;

    const sparkAll = new SparkMD5.ArrayBuffer();
    const spark = new SparkMD5.ArrayBuffer();

    // 分片的 Md5
    const chunkFileReader = new FileReader();
    chunkFileReader.onerror = function () {
      console.warn('oops, something went wrong.');
    };
    chunkFileReader.onload = function (e) {
      sparkAll.append(e.target!!.result);
      spark.append(e.target!!.result);
      params.chunks.push({
        chunk: currentChunk + 1,
        start: currentChunk * chunkSize, // 计算分片的起始位置
        end:
          currentChunk * chunkSize + chunkSize >= file.size
            ? file.size
            : currentChunk * chunkSize + chunkSize, // 计算分片的结束位置
        chunkMd5: spark.end(null),
        chunks,
        currentBuffer: e.target!!.result,
      });

      currentChunk++; // 累加分片

      // 文件删除了，就不要在继续计算了
      const d = workStatusList.current.filter((it) => {
        return it.uid == file.uid;
      });
      if (d.length == 0) return;
      if (!preStatusRef.current) return;

      if (currentChunk < chunks) {
        workStatusList.current = workStatusList.current.map((it) => {
          if (it.uid == file.uid) it.preProgress = ((currentChunk / chunks) * 100).toFixed(2);
          return it;
        });
        setWorkList([
          ...workStatusList.current.map((it) => {
            return { uid: it.uid, data: it.data };
          }),
        ]);
        loadNext();
      } else {
        params.file.fileChunks = params.chunks.length;
        params.file.fileMd5 = sparkAll.end(null);
        handleAddUploadTask({
          name: file.name,
          size: file.size,
          chunks: params.file.fileChunks,
          md5: params.file.fileMd5,
        }).then((ut_id) => {
          if (!ut_id) return;
          workStatusList.current = workStatusList.current.map((it) => {
            if (it.uid == file.uid) {
              it.status = 'uploading';
              it.preProgress = '100';
              it.data.md5 = params.file.fileMd5;
              it.ut_id = ut_id;
              it.params = params;
            }
            return it;
          });
          setWorkList([
            ...workStatusList.current.map((it) => {
              return { uid: it.uid, data: it.data };
            }),
          ]);
          setUploadPace(1 - uploadPace);
        });
      }
    };

    // 兼容性的处理
    const blobSlice = File.prototype.slice || File.prototype.mozSlice || File.prototype.webkitSlice;

    function loadNext() {
      const start = currentChunk * chunkSize;
      const end = start + chunkSize >= file.size ? file.size : start + chunkSize;
      chunkFileReader.readAsArrayBuffer(blobSlice.call(file, start, end));
    }

    loadNext();
  }

  return (
    <PageContainer
      extra={[
        <Button key="back" type="primary" onClick={() => history.goBack()}>
          返回
        </Button>,
      ]}
    >
      <ProCard
        actions={[
          <Button key="reset" type="link" icon={<DeleteOutlined />} onClick={reset} block>
            清空
          </Button>,
        ]}
      >
        <Dragger {...props}>
          <p className="ant-upload-drag-icon">
            <InboxOutlined />
          </p>
          <p className="ant-upload-text">单击或将文件拖动到此区域可以上传文件</p>
          <p className="ant-upload-hint">文件大小对上传速度有显著影响，请耐心等待上传完成。</p>
        </Dragger>
      </ProCard>
    </PageContainer>
  );
};

export default FragmentUpload;
